@@ -203,6 +203,8 @@ namespace eval ::annotate {
       foreach logline $log_lines {
         annotate_color_granny $w.text $now $logline
       }
+      $w.text yview moveto 0
+      update
       $w.text configure -state disabled
       bind $w.bottom.dayentry <Return> [namespace code {redo $w}]
       $w.bottom.redo configure -command [namespace code {redo $w}]
