@@ -3,7 +3,7 @@
 #
  
 # 
-# $Id: commit.tcl,v 1.21 2003/11/04 07:14:35 dorothyr Exp $
+# $Id: commit.tcl,v 1.22 2004/11/14 07:07:27 dorothyr Exp $
 #
 # Set up a small commit dialog.
 # 
@@ -26,6 +26,12 @@ proc commit_run {} {
   # to be committed.
   set cvsglb(commit_list) [workdir_list_files]
 
+  # If we want to use an external editor, just do it
+  if {$cvscfg(use_cvseditor)} {
+    cvs_commit "" "" $cvsglb(commit_list)
+    return
+  }
+
   if {[winfo exists .commit]} {
     wm deiconify .commit
     raise .commit
@@ -104,30 +110,17 @@ proc commit_run {} {
       wm withdraw .commit
     }
  
- 
-  if {$cvscfg(use_cvseditor)} {
-    .commit.lcomment configure \
-      -text "\nAn editor will be invoked so that you can enter a log message.\n" \
-      -anchor c
-    .commit.ok configure -text "Start the Editor"
-    .commit.quit configure -text "Cancel"
-    pack .commit.lcomment -in .commit.comment \
-      -side left -fill x -expand y
-    pack .commit.ok .commit.quit -in .commit.down \
-      -side left -ipadx 2 -ipady 2 -padx 4 -pady 4 -fill both -expand 1
-  } else {
-    .commit.lcomment configure -text "Your log message" \
-      -anchor w
-    .commit.ok configure -text "OK"
-    .commit.quit configure -text "Close"
-    pack .commit.lcomment -in .commit.comment \
-      -side left -fill x -pady 3
-    pack .commit.tcomment -in .commit.comment \
-      -side left -fill both -expand 1 -pady 3
+  .commit.lcomment configure -text "Your log message" \
+    -anchor w
+  .commit.ok configure -text "OK"
+  .commit.quit configure -text "Close"
+  pack .commit.lcomment -in .commit.comment \
+    -side left -fill x -pady 3
+  pack .commit.tcomment -in .commit.comment \
+    -side left -fill both -expand 1 -pady 3
 
-    pack .commit.ok .commit.apply .commit.clear .commit.quit -in .commit.down \
-      -side left -ipadx 2 -ipady 2 -padx 4 -pady 4 -fill both -expand 1
-  }
+  pack .commit.ok .commit.apply .commit.clear .commit.quit -in .commit.down \
+    -side left -ipadx 2 -ipady 2 -padx 4 -pady 4 -fill both -expand 1
   # May be needed for slower framebuffers, but it doesn't work with
   # some window managers (fvwm 2.4 and tvtwm are known)
   #tkwait visibility .commit
