@@ -3,7 +3,7 @@
 #
 
 #
-# $Id: cvs.tcl,v 1.127 2004/03/18 06:36:29 dorothyr Exp $
+# $Id: cvs.tcl,v 1.129 2004/11/15 06:04:24 dorothyr Exp $
 # 
 # Contains procedures used in interaction with CVS.
 #
@@ -36,13 +36,16 @@ proc cvs_sandbox_runcmd {cmd output_var} {
   gen_log:log T "ENTER ($cmd $output_var)"
   set pid [pid]
   
+  if {! [file isdirectory $cvscfg(tmpdir)]} {
+    gen_log:log F "MKDIR $cvscfg(tmpdir)"
+    file mkdir $cvscfg(tmpdir)
+  }
   cd $cvscfg(tmpdir)
   gen_log:log F "CD [pwd]"
   if {! [file isdirectory cvstmpdir.$pid]} {
     gen_log:log F "MKDIR cvstmpdir.$pid"
     file mkdir cvstmpdir.$pid
   }
-
   cd cvstmpdir.$pid
   gen_log:log F "CD [pwd]"
 
@@ -66,7 +69,7 @@ proc cvs_sandbox_filetags {mcode filenames} {
   gen_log:log T "ENTER ($mcode $filenames)"
   
   cd [file join $cvscfg(tmpdir) cvstmpdir.$pid $mcode]
-  set commandline "$cvs -n log -l $filenames"
+  set commandline "$cvs log $filenames"
   gen_log:log C "$commandline"
   set ret [catch {eval "exec $commandline"} view_this]
   if {$ret} {
@@ -586,7 +589,7 @@ proc cvs_logcanvas {directory files} {
   }
 
   foreach file $files {
-    ::logcanvas::new $directory $file "diff ok" "$cvs -n log -l \"$file\""
+    ::logcanvas::new $directory $file "diff ok" "$cvs log \"$file\""
   }
 
   gen_log:log T "LEAVE"
@@ -838,7 +841,7 @@ proc cvs_commit {revision comment args} {
   gen_log:log T "LEAVE"
 }
 
-proc cvs_tag {tagname force branch args} {
+proc cvs_tag {tagname force branch update args} {
 #
 # This tags a file in a directory.
 #
@@ -846,7 +849,7 @@ proc cvs_tag {tagname force branch args} {
   global cvscfg
   global incvs
 
-  gen_log:log T "ENTER ($tagname $force $branch $args)"
+  gen_log:log T "ENTER ($tagname $force $branch $update $args)"
 
   if {! $incvs} {
     cvs_notincvs
@@ -874,7 +877,7 @@ proc cvs_tag {tagname force branch args} {
   $v\::do "$command"
   $v\::wait
 
-  if {$branch == "yes"} {
+  if {$update == "yes"} {
     # update so we're on the branch
     set command "$cvs update -r $tagname $filelist"
     #gen_log:log C "$command"
@@ -1245,7 +1248,7 @@ proc cvs_filelog {filename} {
     return
   }
 
-  set commandline "$cvs -d $cvscfg(cvsroot) -n log -l \"$filename\""
+  set commandline "$cvs -d $cvscfg(cvsroot) log \"$filename\""
 
   # Log canvas viewer
   logcanvas::new [pwd] $filename "no file" $commandline
@@ -1474,7 +1477,7 @@ proc cvs_gettaglist {filename} {
     return $keepers
   }
 
-  set commandline "$cvs -d $cvscfg(cvsroot) -n log -l \"$filename\""
+  set commandline "$cvs -d $cvscfg(cvsroot) log \"$filename\""
   gen_log:log C "$commandline"
   set ret [catch {eval "exec $commandline"} view_this]
   if {$ret} {
@@ -1585,3 +1588,56 @@ proc cvs_rtag { cvsroot mcode branch force oldtag newtag } {
   gen_log:log T "LEAVE"
 }
 
+proc cvs_ascii { args } {
+# This converts a binary file to ASCII
+  global cvs
+  global cvscfg
+  global incvs
+  global cvsglb
+
+  gen_log:log T "ENTER ($args)"
+  if {! $incvs} {
+    cvs_notincvs
+    return 1
+  }
+  set filelist [join $args]
+
+  gen_log:log D "Changing sticky flag"
+  gen_log:log D "$cvs admin -kkv $filelist"
+  set cmd [exec::new "$cvs admin -kkv $filelist"]
+  # gen_log:log D "Updating file list"
+  # set cmd [exec::new "$cvs update $filelist"]
+  if {$cvscfg(auto_status)} {
+    $cmd\::wait
+    setup_dir
+  }
+
+  gen_log:log T "LEAVE"
+}
+
+proc cvs_binary { args } {
+# This converts an ASCII file to binary
+  global cvs
+  global cvscfg
+  global incvs
+  global cvsglb
+
+  gen_log:log T "ENTER ($args)"
+  if {! $incvs} {
+    cvs_notincvs
+    return 1
+  }
+  set filelist [join $args]
+
+  gen_log:log D "Changing sticky flag"
+  gen_log:log D "$cvs admin -kb $filelist"
+  set cmd [exec::new "$cvs admin -kb $filelist"]
+  # gen_log:log D "Updating file list"
+  # set cmd [exec::new "$cvs update $filelist"]
+  if {$cvscfg(auto_status)} {
+    $cmd\::wait
+    setup_dir
+  }
+
+  gen_log:log T "LEAVE"
+}
