@@ -3,7 +3,7 @@
 #
 
 #
-# $Id: dialog.tcl,v 1.34 2004/04/20 04:50:45 dorothyr Exp $
+# $Id: dialog.tcl,v 1.35 2004/11/08 04:39:10 dorothyr Exp $
 #
 # Smallish dialogs - add, tag
 #
@@ -622,33 +622,45 @@ proc file_tag_dialog {branch} {
   pack .tag.top -side top -fill x
 
   message .tag.top.msg -justify left -aspect 300 -relief groove \
-    -text "Apply a new tag or branch tag\
+    -text "Apply a new tag or branch tag \
            to the marked files, recursively.\
-           Will apply tag to repository and,\
-           if a branch, also update local directory."
+           Will change the repository.\
+           If a branch, it can also update local directory if desired."
 
   label .tag.top.lbl -text "Tag Name" -anchor w
   entry .tag.top.entry -relief sunken -textvariable usertagname
-  checkbutton .tag.top.branch -text "Branch tag" \
-     -variable branchflag -onvalue "yes" -offvalue "no"
-  checkbutton .tag.top.force -text "Move existing" \
+  checkbutton .tag.top.branch -text "Branch tag (-b)" \
+     -variable branchflag -onvalue "yes" -offvalue "no" \
+     -command { 
+        if {$branchflag == "no"} {\
+           .tag.mid.upd config -state disabled; set updflag "no" } \
+        else {.tag.mid.upd config -state normal } \
+      }
+  checkbutton .tag.top.force -text "Move existing (-F)" \
      -variable forceflag -onvalue "yes" -offvalue "no"
 
+  frame .tag.mid -relief groove -bd 2
+  checkbutton .tag.mid.upd -text "Update current directory to be on the new tag" \
+      -variable updflag -onvalue "yes" -offvalue "no"
+
   grid columnconf .tag.top 1 -weight 1
   grid rowconf .tag.top 3 -weight 1
   grid .tag.top.msg -column 0 -row 0 -columnspan 2 -pady 2 -sticky ew
   grid .tag.top.lbl -column 0 -row 1 -sticky nw
-  grid .tag.top.entry -column 1 -row 1
+  grid .tag.top.entry -column 1 -row 1 -sticky ew
   grid .tag.top.branch -column 1 -row 2 -sticky w
   grid .tag.top.force -column 1 -row 3 -sticky w
 
+  pack .tag.mid -side top
+  pack .tag.mid.upd
+
   frame .tag.down -relief groove -bd 2
   pack .tag.down -side bottom -fill x -expand 1
   button .tag.down.tag -text "Tag" \
     -command {
       grab release .tag
       destroy .tag
-      cvs_tag $usertagname $forceflag $branchflag [workdir_list_files]
+      cvs_tag $usertagname $forceflag $branchflag $updflag [workdir_list_files]
     }
   button .tag.down.cancel -text "Cancel" \
     -command { grab release .tag; destroy .tag }
@@ -656,6 +668,13 @@ proc file_tag_dialog {branch} {
   pack .tag.down.tag .tag.down.cancel -in .tag.down -side left \
     -ipadx 2 -ipady 2 -padx 4 -pady 4 -fill both -expand 1
 
+  if {$branchflag == "no"} {
+     .tag.mid.upd config -state disabled
+     set updflag "no"
+  } else {
+     .tag.mid.upd config -state normal
+  }
+
   wm title .tag "tag"
   wm minsize .tag 1 1
   gen_log:log T "LEAVE"
@@ -687,9 +706,9 @@ proc rtag_dialog { cvsroot module branch } {
     label .modtag.top.nlbl -text "New Tag" -anchor w
     entry .modtag.top.nentry -textvariable [namespace current]::ntag \
       -relief sunken
-    checkbutton .modtag.top.branch -text "Branch tag" \
+    checkbutton .modtag.top.branch -text "Branch tag (-b)" \
        -variable [namespace current]::branch -onvalue "yes" -offvalue "no"
-    checkbutton .modtag.top.force -text "Move existing" \
+    checkbutton .modtag.top.force -text "Move existing (-F)" \
        -variable [namespace current]::force -onvalue "yes" -offvalue "no"
 
     grid columnconf .modtag.top 1 -weight 1
