@@ -2,7 +2,7 @@
 # Columns for listing CVS files and their status
 #
 #
-# $Id: dircanvas.tcl,v 1.19 2002/01/21 08:19:58 dorothyr Exp $
+# $Id: dircanvas.tcl,v 1.41 2003/11/04 07:21:12 dorothyr Exp $
 #
 
 proc DirCanvas:create {w args} {
@@ -10,6 +10,7 @@ proc DirCanvas:create {w args} {
   global cvsglb
   global arr
   global incvs
+  global inrcs
 
   gen_log:log T "ENTER ($w $args)"
 
@@ -54,7 +55,9 @@ proc DirCanvas:create {w args} {
   DirCanvas:column $w filecol "file"
   DirCanvas:column $w statcol "status"
   DirCanvas:column $w datecol "date"
-  if {$incvs} {
+gen_log:log D "**** incvs $incvs  inrcs $inrcs ****"
+  if {$incvs || $inrcs} {
+gen_log:log D "**** going to make wrevcol ****"
     DirCanvas:column $w wrevcol "revision"
     DirCanvas:column $w editcol "editors"
   }
@@ -76,7 +79,7 @@ proc DirCanvas:create {w args} {
   set col [lindex $cvsglb(sort_pref) 0]
   set sense [lindex $cvsglb(sort_pref) 1]
   gen_log:log D "$incvs  $col"
-  if { (! $incvs )  && ( $col == "editcol" || $col == "wrevcol") } {
+  if { (! ($incvs || $inrcs))  && ( $col == "editcol" || $col == "wrevcol") } {
     gen_log:log T "setting sort to column \"filecol!\""
     set col "filecol"
     set sense "-decreasing"
@@ -107,6 +110,7 @@ proc DirCanvas:column {w column headtext} {
   global cvscfg
   global cvsglb
   global incvs
+  global inrcs
   global arr
 
   gen_log:log T "ENTER ($w $column headtext)"
@@ -120,6 +124,10 @@ proc DirCanvas:column {w column headtext} {
   bind $w.$column.list <B2-Motion> "DirCanvas:drag_windows $w %W %y"
   bind $w.$column.list <MouseWheel> \
       "DirCanvas:scroll_windows $w scroll \[expr {-(%D/120)*4}\] units"
+  bind $w.$column.list <ButtonPress-4> \
+      "DirCanvas:scroll_windows $w scroll -1 units"
+  bind $w.$column.list <ButtonPress-5> \
+      "DirCanvas:scroll_windows $w scroll 1 units"
 
   frame $w.$column.head -relief raised -bd 2
   label $w.$column.head.lbl -text "$headtext"
@@ -138,7 +146,7 @@ proc DirCanvas:column {w column headtext} {
   pack $w.$column.xscroll -side bottom -fill x
 
   if {$column == "statcol"} {
-    if {$cvscfg(showstatcol)} {
+    if {($incvs || $inrcs) && $cvscfg(showstatcol)} {
       DirCanvas:map_column $w statcol
     } else {
       gen_log:log T "LEAVE (skipping statcol)"
@@ -154,7 +162,7 @@ proc DirCanvas:column {w column headtext} {
     return
   }
   if {$column == "editcol"} {
-    if {$incvs && $cvscfg(econtrol)} {
+    if {($incvs || $inrcs) && $cvscfg(econtrol)} {
       DirCanvas:map_column $w editcol
     } else {
       gen_log:log T "LEAVE (skipping editcol)"
@@ -166,7 +174,7 @@ proc DirCanvas:column {w column headtext} {
 
 proc DirCanvas:map_column {w column} {
 
-  gen_log:log T "ENTER ($w $column)"
+  #gen_log:log T "ENTER ($w $column)"
   set mapped_columns [pack slaves $w]
   gen_log:log D "$mapped_columns"
 
@@ -189,6 +197,8 @@ proc DirCanvas:map_column {w column} {
   pack $w.$column.head.sbut -side right
   pack $w.$column.head.lbl -side right -fill x -expand yes
   pack $w.$column.list -side top -fill both -ipadx 2 -expand yes
+
+  #gen_log:log T "LEAVE"
 }
 
 #
@@ -212,10 +222,10 @@ proc DirCanvas:newitem {w f} {
   global Filelist
   global cvsglb
 
-  gen_log:log T "ENTER ($w $f)"
+  #gen_log:log T "ENTER ($w $f)"
 
   set DirList($w:$f:name) $f
-  gen_log:log D "Newitem $f status $Filelist($f:status)"
+  #gen_log:log D "Newitem $f status $Filelist($f:status)"
   set DirList($w:$f:status) $Filelist($f:status)
   set DirList($w:$f:date) $Filelist($f:date)
   set DirList($w:$f:sticky) $Filelist($f:stickytag)
@@ -238,6 +248,8 @@ proc DirCanvas:loadimages { } {
     -format gif -file [file join $cvscfg(bitmapdir) paper.gif]
   image create photo cvsdir \
     -format gif -file [file join $cvscfg(bitmapdir) cvsdir.gif]
+  image create photo rcsdir \
+    -format gif -file [file join $cvscfg(bitmapdir) rcsdir.gif]
   image create photo folder \
     -format gif -file [file join $cvscfg(bitmapdir) folder.gif]
 
@@ -263,6 +275,10 @@ proc DirCanvas:loadimages { } {
     -format gif -file [file join $cvscfg(bitmapdir) stat_minus.gif]
   image create photo stat_conf \
     -format gif -file [file join $cvscfg(bitmapdir) stat_conf.gif]
+  image create photo stat_rcs \
+    -format gif -file [file join $cvscfg(bitmapdir) stat_rcs.gif]
+  image create photo stat_rcs_mod \
+    -format gif -file [file join $cvscfg(bitmapdir) stat_rcs_mod.gif]
 }
 
 #
@@ -289,23 +305,21 @@ proc DirCanvas:deltree {w} {
   }
 }
 
-proc DirCanvas:flash {widg y} {
+proc DirCanvas:flash {w y} {
   global cvsglb
 
-  $widg.tx$y configure -relief raised
+  DirCanvas:setTextHBox $w $w.filecol.list.tx$y
 }
 
-proc DirCanvas:unflash {widg y} {
-  global cvsglb
-
-  set savfg [lindex [$widg.tx$y configure -foreground] 4]
-  set savbg [lindex [$widg.tx$y configure -background] 4]
-  set relief [lindex [$widg.tx$y configure -relief] 4]
-  $widg.tx$y configure -fg $savfg -bg $savbg
-  if {$savbg != $cvsglb(hlbg)} {
-    $widg.tx$y configure -relief flat
+proc DirCanvas:unflash {w y f} {
+  global DirList
+  
+  # Don't unflash if this is one that is selected:
+  if { ! $DirList($w:$f:selected) } {
+  	DirCanvas:clearTextHBox $w $w.filecol.list.tx$y
   }
-}
+
+ }
 
 #
 # Change the selection to the indicated item
@@ -316,13 +330,12 @@ proc DirCanvas:setselection {w y f} {
 
   gen_log:log T "ENTER ($w $y $f)"
 
-  DirCanvas:unselectall $w
+  DirCanvas:unselectall $w 1
   gen_log:log D "adding \"$f\""
   set DirList($w:$f:selected) 1
   set DirList($w:selection) [list "$f"]
   set cvsglb(current_selection) $DirList($w:selection)
-  $w.filecol.list.tx$y configure -relief raised \
-    -bg $cvsglb(hlbg) -fg $cvsglb(hlfg)
+  DirCanvas:setTextHBox $w $w.filecol.list.tx$y
 
   # This seems to be necessary to enable some OS's (esp. Mandrake)
   # to get a double-click
@@ -337,38 +350,63 @@ proc DirCanvas:addselection {w y f} {
 
   gen_log:log T "ENTER ($w $y $f)"
 
+  regsub -all {\%\%} $f {%} fn
+  regsub -all {\\\$} $fn {$} fn
   # If it's already selected, unselect it
-  if { $DirList($w:$f:selected) } {
-    gen_log:log D "\"$f\" was selected - unselecting"
-    set DirList($w:$f:selected) 0
-    set idx [lsearch -exact $DirList($w:selection) "$f"]
+  if { $DirList($w:$fn:selected) } {
+    gen_log:log D "\"$fn\" was selected - unselecting"
+    set DirList($w:$fn:selected) 0
+    set idx [lsearch -exact $DirList($w:selection) "$fn"]
     if {$idx > -1} {
-      gen_log:log D "found \"$f\" - removing from selection list"
+      gen_log:log D "found \"$fn\" - removing from selection list"
       set DirList($w:selection) [lreplace $DirList($w:selection) $idx $idx]
       gen_log:log D "$DirList($w:selection)"
-      $w.filecol.list.tx$y configure -relief flat \
-        -bg $cvsglb(canvbg) -fg $cvsglb(fg) \
+      DirCanvas:clearTextHBox $w $w.filecol.list.tx$y
     }
   } else {
-    gen_log:log D "adding \"$f\""
-    $w.filecol.list.tx$y configure -relief raised \
-      -bg $cvsglb(hlbg) -fg $cvsglb(hlfg)
-    set DirList($w:$f:selected) 1
-    lappend DirList($w:selection) "$f"
+    gen_log:log D "adding \"$fn\""
+    DirCanvas:setTextHBox $w $w.filecol.list.tx$y
+    set DirList($w:$fn:selected) 1
+    lappend DirList($w:selection) "$fn"
   }
   set cvsglb(current_selection) $DirList($w:selection)
   gen_log:log D "selection is \"$cvsglb(current_selection)\""
   gen_log:log T "LEAVE"
 }
 
+
+# clear any text highlight box (used by set/clearselection)
+proc DirCanvas:clearTextHBox {w id} {
+   global cvsglb
+
+    # clear the tag corresponding to the text label
+    catch {$w.filecol.list delete HBox$id}
+    $w.filecol.list itemconfigure $id -fill $cvsglb(textfg)
+}
+
+# set a text highligh box (used by set/clearselection)
+proc DirCanvas:setTextHBox {w id} {
+   global cvsglb
+   
+   # get the bounding box for the text id
+   set bbox [$w.filecol.list bbox $id]
+   if {[llength $bbox]==4} {
+    # create rectangle with fill, tagged with the same ID as the text, so we can delete it later
+    set i [eval $w.filecol.list create rectangle $bbox -fill $cvsglb(hlbg) -tag HBox$id -outline \"\"] 
+    
+    $w.filecol.list itemconfigure $id -fill $cvsglb(hlfg)
+    $w.filecol.list lower $i
+  } 
+}
+
+
 proc DirCanvas:addrange {w y f} {
   global DirList
   global cvsglb
 
   gen_log:log T "ENTER ($w $y $f)"
   if {! [info exists DirList($w:selection)] || [llength $DirList($w:selection)] < 1} {
-    $w.filecol.list.tx$y configure -relief raised \
-      -bg $cvsglb(hlbg) -fg $cvsglb(hlfg)
+    DirCanvas:clearTextHBox $w $w.filecol.list.tx$y
     set DirList($w:$f:selected) 1
     lappend DirList($w:selection) "$f"
     set cvsglb(current_selection) $DirList($w:selection)
@@ -386,8 +424,7 @@ proc DirCanvas:addrange {w y f} {
       set jy $DirList($w:$j:y)
       if { $jy > $iy && $y > $jy} {
         gen_log:log D "$j y=$jy"
-        $w.filecol.list.tx$jy configure -relief raised \
-          -bg $cvsglb(hlbg) -fg $cvsglb(hlfg)
+        DirCanvas:setTextHBox $w $w.filecol.list.tx$jy
         set DirList($w:$j:selected) 1
         if {[lsearch -exact $DirList($w:selection) $j] == -1} {
           lappend DirList($w:selection) "$j"
@@ -400,8 +437,7 @@ proc DirCanvas:addrange {w y f} {
       set jy $DirList($w:$j:y)
       if { $jy < $iy && $y < $jy} {
         gen_log:log D "$j y=$jy"
-        $w.filecol.list.tx$jy configure -relief raised \
-          -bg $cvsglb(hlbg) -fg $cvsglb(hlfg)
+        DirCanvas:setTextHBox $w $w.filecol.list.tx$jy
         set DirList($w:$j:selected) 1
         if {[lsearch -exact $DirList($w:selection) $j] == -1} {
           lappend DirList($w:selection) "$j"
@@ -410,8 +446,7 @@ proc DirCanvas:addrange {w y f} {
     }
   }
 
-  $w.filecol.list.tx$y configure -relief raised \
-    -bg $cvsglb(hlbg) -fg $cvsglb(hlfg)
+  DirCanvas:setTextHBox $w $w.filecol.list.tx$y
   set DirList($w:$f:selected) 1
   if {[lsearch -exact $DirList($w:selection) $f] == -1} {
     lappend DirList($w:selection) "$f"
@@ -422,35 +457,41 @@ proc DirCanvas:addrange {w y f} {
   gen_log:log T "LEAVE"
 }
 
-proc DirCanvas:unselectall {w} {
+proc DirCanvas:unselectall {w force} {
   global DirList
   global cvsglb
 
   gen_log:log T "ENTER ($w)"
-  foreach s [array names DirList $w:*:name] {
-    set f $DirList($s)
-    set y $DirList($w:$f:y)
-    set DirList($w:$f:selected) 0
-    $w.filecol.list.tx$y configure -relief flat \
-      -bg $cvsglb(canvbg) -fg $cvsglb(fg)
-    # Prepending ./ to the filename prevents tilde expansion
-    if {! [file exists ./$f]} {
-      # If a file CVS knows about doesn't exist, write its name in light ink
-      $w.filecol.list.tx$y configure -fg $cvsglb(dfg)
+ # Don't clear unless we are'nt over anything
+  if { $force || [ $w.filecol.list gettags current ] == "" } {
+
+    foreach s [array names DirList $w:*:name] {
+      set f $DirList($s)
+      set y $DirList($w:$f:y)
+      set DirList($w:$f:selected) 0
+      DirCanvas:clearTextHBox $w $w.filecol.list.tx$y
+      # Prepending ./ to the filename prevents tilde expansion
+      if {! [file exists ./$f]} {
+	# If a file CVS knows about doesn't exist, write its name in light ink
+	$w.filecol.list itemconfigure $w.filecol.list.tx$y -fill $cvsglb(dfg)
+      }
     }
+    set DirList($w:selection) {}
+    set cvsglb(current_selection) {}
   }
-  set DirList($w:selection) {}
-  set cvsglb(current_selection) {}
   gen_log:log T "LEAVE"
 }
 
 proc DirCanvas:areaStart {w x y} {
   global areaX1 areaY1
+  global areaX2 areaY2
 
   gen_log:log T "ENTER ($w $x $y)"
   $w delete area
   set areaX1 [$w canvasx $x]
   set areaY1 [$w canvasy $y]
+  set areaX2 $areaX1
+  set areaY2 $areaY1
 }
 
 proc DirCanvas:areaStroke {w x y} {
@@ -488,11 +529,10 @@ proc DirCanvas:areaFind {w} {
   set parent [winfo parent [winfo parent $w]]
   foreach i $items {
     set itags [$w gettags $i]
-    if {[lindex $itags 0] == "x"} {
+    if { [string match .workdir* [lindex $itags 0]]  } {
       set iy [lindex $itags 1]
       gen_log:log D "$w tx$iy"
-      gen_log:log D [array names DirList *$iy]
-      DirCanvas:addselection $parent $iy $DirList($w:$iy)
+      DirCanvas:addselection $parent $iy [lindex $itags 2]
     }
   }
   $w delete area
@@ -506,6 +546,7 @@ proc DirCanvas:build {w} {
   global cvscfg
   global cvsglb
   global incvs
+  global inrcs
 
   gen_log:log T "ENTER ($w)"
   set flist $w.filecol.list
@@ -541,7 +582,7 @@ proc DirCanvas:build {w} {
 
   set sortcol [lindex $cvsglb(sort_pref) 0]
   set sortsense [lindex $cvsglb(sort_pref) 1]
-  if { (! $incvs )  && ( $sortcol == "editcol" || $sortcol == "wrevcol") } {
+  if { (!($incvs || $inrcs))  && ( $sortcol == "editcol" || $sortcol == "wrevcol") } {
     gen_log:log T "setting sort to column \"filecol!\""
     set sortcol "filecol"
     set sortsense "-decreasing"
@@ -558,21 +599,23 @@ proc DirCanvas:build {w} {
   }
 
   set itemlist {}
+  # Always sort by name first
+  set sortedlist [lsort $sortsense -index 0 $AllColumns]
   switch -- $sortcol {
    "filecol" {
-    set sortedlist [lsort $sortsense -index 0 $AllColumns]
+    # Only by name
    }
    "statcol" {
-    set sortedlist [lsort $sortsense -index 1 $AllColumns]
+    set sortedlist [lsort $sortsense -index 1 $sortedlist]
    }
    "datecol" {
-    set sortedlist [lsort $sortsense -index 2 $AllColumns]
+    set sortedlist [lsort $sortsense -index 2 $sortedlist]
    }
    "wrevcol" {
-    set sortedlist [lsort -dictionary $sortsense -index 3 $AllColumns]
+    set sortedlist [lsort -dictionary $sortsense -index 3 $sortedlist]
    }
    "editcol" {
-    set sortedlist [lsort $sortsense -index 4 $AllColumns]
+    set sortedlist [lsort $sortsense -index 4 $sortedlist]
    }
   }
 
@@ -628,16 +671,16 @@ proc DirCanvas:build {w} {
       }
      "<directory>" {
        set DirList($w:$f:icon) folder
-       if {$f == ".."} {
-         set DirList($w:$f:popup) updir_pop
-       } else {
-         set DirList($w:$f:popup) folder_pop
-       }
+       set DirList($w:$f:popup) folder_pop
       }
      "<directory:CVS>" {
        set DirList($w:$f:icon) cvsdir
        set DirList($w:$f:popup) cvsdir_pop
       }
+     "<directory:RCS>" {
+       set DirList($w:$f:icon) rcsdir
+       set DirList($w:$f:popup) rcs_pop
+      }
      "Up-to-date" {
        set DirList($w:$f:icon) stat_ok
        set DirList($w:$f:popup) stat_ok_pop
@@ -685,6 +728,18 @@ proc DirCanvas:build {w} {
        set DirList($w:$f:icon) stat_ques
        set DirList($w:$f:popup) paper_pop
       }
+      "RCS" {
+       set DirList($w:$f:icon) stat_rcs
+       set DirList($w:$f:popup) rcs_pop
+      }
+      "RCS Modified" {
+       set DirList($w:$f:icon) stat_rcs_mod
+       set DirList($w:$f:popup) rcs_pop
+      }
+      "RCS Needs Checkout" {
+       set DirList($w:$f:icon) stat_ex
+       set DirList($w:$f:popup) rcs_pop
+      }
       default {
        set DirList($w:$f:icon) paper
        set DirList($w:$f:popup) paper_pop
@@ -692,19 +747,20 @@ proc DirCanvas:build {w} {
     }
 
     # Easy way to unselect everything by clicking in a blank area
-    bind $flist <1> "DirCanvas:unselectall $w"
+    bind $flist <1> "DirCanvas:unselectall $w 0"
     bind $flist <Shift-1> " "
 
     bind $flist <2> "DirCanvas:areaStart $flist %x %y"
     bind $flist <B2-Motion> "DirCanvas:areaStroke $flist %x %y; \
                              DirCanvas:drag_windows $w %W %y"
-    bind $flist <ButtonRelease-2> "DirCanvas:areaFind $flist"
-    bind $flist <3> "DirCanvas:unselectall $w; \
-                     DirCanvas:areaStart $flist %x %y"
+    bind $flist <B2-Motion><ButtonRelease-2> "DirCanvas:unselectall $w 1; \
+                                              DirCanvas:areaFind $flist"
+    bind $flist <3> "DirCanvas:areaStart $flist %x %y"
     bind $flist <Shift-3> "DirCanvas:areaStart $flist %x %y"
     bind $flist <B3-Motion> "DirCanvas:areaStroke $flist %x %y; \
                              DirCanvas:drag_windows $w %W %y"
-    bind $flist <ButtonRelease-3> "DirCanvas:areaFind $flist"
+    bind $flist <B3-Motion><ButtonRelease-3> "DirCanvas:unselectall $w 1; \
+                                              DirCanvas:areaFind $flist"
     # In the bindings, filenames need any single percents replaced with
     # double to avoid interpretation as an event field
     regsub -all {\%} $f {%%} fn
@@ -712,33 +768,31 @@ proc DirCanvas:build {w} {
 
     # The "x" tag is used for area selection.
     # Draw the icon
-    label $flist.ic$y -image $DirList($w:$f:icon) -padx 0 -pady 0 \
-      -borderwidth 1 -height $yincr -bg $cvsglb(canvbg)
-    $flist create window $x $y -window $flist.ic$y \
-      -anchor w -tags [list x $y]
-    bind $flist.ic$y <1> "DirCanvas:setselection $w $y \"$fn\""
-    bind $flist.ic$y <Enter> "DirCanvas:flash $flist $y"
-    bind $flist.ic$y <Leave> "DirCanvas:unflash $flist $y"
-    bind $flist.ic$y <Shift-1> "DirCanvas:addrange $w $y \"$fn\""
-    bind $flist.ic$y <Control-1> "DirCanvas:addselection $w $y \"$fn\""
-    bind $flist.ic$y <Double-1> {workdir_edit_file [workdir_list_files]}
-    bind $flist.ic$y <2> "DirCanvas:popup $flist $y %X %Y \"$fn\""
-    bind $flist.ic$y <3> "DirCanvas:popup $flist $y %X %Y \"$fn\""
+     set k [$flist create image $x $y -image $DirList($w:$f:icon) \
+       -anchor w -tags [list x $y] ]
+     $flist bind $k <1> "DirCanvas:setselection $w $y \"$fn\""
+     $flist bind $k <Shift-1> "DirCanvas:addrange $w $y \"$fn\""
+     $flist bind $k <Control-1> "DirCanvas:addselection $w $y \"$fn\""
+     $flist bind $k <Double-1> {workdir_edit_file [workdir_list_files]}
+     $flist bind $k <2> "DirCanvas:areaStart $flist %x %y; \
+                         DirCanvas:popup $flist $y %X %Y \"$fn\""
+     $flist bind $k <3> "DirCanvas:areaStart $flist %x %y; \
+                         DirCanvas:popup $flist $y %X %Y \"$fn\""
 
     # Draw the label
-    label $flist.tx$y -text $f -font $cvscfg(listboxfont) -padx 0 -pady 0 \
-      -borderwidth 1 -anchor w -bg $cvsglb(canvbg) -fg $lblfg
-    $flist create window $lblx $y -window $flist.tx$y \
-      -anchor w -tags [list x $y lbl]
+     $flist create text $lblx $y  -text $f -font $cvscfg(listboxfont) \
+       -anchor w -tags [list $flist.tx$y $y $fn] -fill $lblfg
+     $flist bind $flist.tx$y <1> "DirCanvas:setselection $w $y \"$fn\""
+     $flist bind $flist.tx$y <Shift-1> "DirCanvas:addrange $w $y \"$fn\""
+     $flist bind $flist.tx$y <Enter> "DirCanvas:flash $w $y"
+     $flist bind $flist.tx$y <Leave> "DirCanvas:unflash $w $y \"$fn\""
+     $flist bind $flist.tx$y <Control-1> "DirCanvas:addselection $w $y \"$fn\""
+     $flist bind $flist.tx$y <Double-1> {workdir_edit_file [workdir_list_files]}
+     $flist bind $flist.tx$y <2> "DirCanvas:areaStart $flist %x %y; \
+                                  DirCanvas:popup $flist $y %X %Y \"$fn\""
+     $flist bind $flist.tx$y <3> "DirCanvas:areaStart $flist %x %y; \
+                                  DirCanvas:popup $flist $y %X %Y \"$fn\""
 
-    bind $flist.tx$y <1> "DirCanvas:setselection $w $y \"$fn\""
-    bind $flist.tx$y <Enter> "DirCanvas:flash $flist $y"
-    bind $flist.tx$y <Leave> "DirCanvas:unflash $flist $y"
-    bind $flist.tx$y <Shift-1> "DirCanvas:addrange $w $y \"$fn\""
-    bind $flist.tx$y <Control-1> "DirCanvas:addselection $w $y \"$fn\""
-    bind $flist.tx$y <Double-1> { workdir_edit_file [workdir_list_files] }
-    bind $flist.tx$y <2> "DirCanvas:popup $flist $y %X %Y \"$fn\""
-    bind $flist.tx$y <3> "DirCanvas:popup $flist $y %X %Y \"$fn\""
     set DirList($w:$f:y) $y
     set DirList($flist:$y) $f
 
@@ -782,31 +836,26 @@ proc DirCanvas:build {w} {
   }
 
   # See which optional columns we need to draw
-  if {$incvs && $cvscfg(showstatcol)} {
-    if {! [winfo ismapped $w.statcol]} {
+  if {$incvs || $inrcs} {
+    if {$cvscfg(showstatcol)} {
       DirCanvas:map_column $w statcol
+    } else {
+      pack forget $w.statcol
+    }
+    if {$cvscfg(econtrol)} {
+      DirCanvas:map_column $w editcol
+    } else {
+      pack forget $w.editcol
     }
-  } else {
-    pack forget $w.statcol
   }
-
   if {$cvscfg(showdatecol)} {
-    if {! [winfo ismapped $w.datecol.list]} {
-      DirCanvas:map_column $w datecol
-    }
+    DirCanvas:map_column $w datecol
   } else {
     pack forget $w.datecol
   }
 
-  if {$incvs && $cvscfg(econtrol)} {
-    if {! [winfo ismapped $w.editcol.list]} {
-      DirCanvas:map_column $w editcol
-    }
-  } else {
-    pack forget $w.editcol
-  }
 
-  set wid [font measure $cvscfg(listboxfont) -displayof $w.filecol $longlbl]
+  set wid [font measure $cvscfg(listboxfont) -displayof $w $longlbl]
   set DirList($w:filecolwidth) [expr {$x + $wid + $lblx + 8}]
   gen_log:log D "width of $flist $maxlbl chars ($wid)"
   # Set a minimum width for the labels.  Otherwise ".." can be hard to select.
@@ -829,34 +878,30 @@ proc DirCanvas:build {w} {
   }
   $flist configure -xscrollcommand "$w.filecol.xscroll set"
 
-  if {$cvscfg(showstatcol)} {
-    set wid [font measure $cvscfg(listboxfont) -displayof $w.statcol $longstat]
-    set DirList($w:statcolwidth) [expr {$wid + 6}]
-    gen_log:log D "statcol width $DirList($w:statcolwidth)"
-    if {$wid < $maxcolwid} {
-      $w.statcol.list configure -width $DirList($w:statcolwidth)
-    } else {
-      gen_log:log D "Reducing statcol width from $wid to $maxcolwid"
-      $w.statcol.list configure -width $maxcolwid
-    }
-    $w.statcol.list configure -xscrollcommand "$w.statcol.xscroll set"
+  set wid [font measure $cvscfg(listboxfont) -displayof $w $longstat]
+  set DirList($w:statcolwidth) [expr {$wid + 6}]
+  gen_log:log D "statcol width $DirList($w:statcolwidth)"
+  if {$wid < $maxcolwid} {
+    $w.statcol.list configure -width $DirList($w:statcolwidth)
+  } else {
+    gen_log:log D "Reducing statcol width from $wid to $maxcolwid"
+    $w.statcol.list configure -width $maxcolwid
   }
+  $w.statcol.list configure -xscrollcommand "$w.statcol.xscroll set"
 
-  if {$cvscfg(showdatecol)} {
-    set wid [font measure $cvscfg(listboxfont) -displayof $w.datecol $longdate]
-    set DirList($w:datecolwidth) [expr {$wid + 6}]
-    if {$wid < $maxcolwid} {
-      $w.datecol.list configure -width $DirList($w:datecolwidth) 
-    } else {
-      gen_log:log D "Reducing datecol width from $wid to $maxcolwid"
-      $w.datecol.list configure -width $maxcolwid
-    }
-    $w.datecol.list configure -xscrollcommand "$w.datecol.xscroll set"
+  set wid [font measure $cvscfg(listboxfont) -displayof $w $longdate]
+  set DirList($w:datecolwidth) [expr {$wid + 6}]
+  if {$wid < $maxcolwid} {
+    $w.datecol.list configure -width $DirList($w:datecolwidth) 
+  } else {
+    gen_log:log D "Reducing datecol width from $wid to $maxcolwid"
+    $w.datecol.list configure -width $maxcolwid
   }
+  $w.datecol.list configure -xscrollcommand "$w.datecol.xscroll set"
 
 
-  if {$incvs} {
-    set wid [font measure $cvscfg(listboxfont) -displayof $w.wrevcol $longtag]
+  if {$incvs || $inrcs} {
+    set wid [font measure $cvscfg(listboxfont) -displayof $w $longtag]
     set DirList($w:revcolwidth) [expr {$wid + 6}]
     gen_log:log D "width of $w.wrevcol $maxtag chars ($wid)"
     # Don't let the column be too wide.
@@ -868,65 +913,75 @@ proc DirCanvas:build {w} {
     }
     $w.wrevcol.list configure -xscrollcommand "$w.wrevcol.xscroll set"
 
-    if {$cvscfg(econtrol)} {
-      set wid [font measure $cvscfg(listboxfont) -displayof $w.editcol $longed]
-      set DirList($w:edcolwidth) [expr {$wid + 6}]
-      gen_log:log D "width of $w.editcol $maxed chars ($wid)"
-      if {$wid < $maxcolwid} {
-        $w.editcol.list configure -width $DirList($w:edcolwidth)
-      } else {
-        gen_log:log D "Reducing editcol width from $wid to $maxcolwid"
-        $w.editcol.list configure -width $maxcolwid
-      }
-      $w.editcol.list configure -xscrollcommand "$w.editcol.xscroll set"
+    set wid [font measure $cvscfg(listboxfont) -displayof $w $longed]
+    set DirList($w:edcolwidth) [expr {$wid + 6}]
+    gen_log:log D "width of $w.editcol $maxed chars ($wid)"
+    if {$wid < $maxcolwid} {
+      $w.editcol.list configure -width $DirList($w:edcolwidth)
+    } else {
+      gen_log:log D "Reducing editcol width from $wid to $maxcolwid"
+      $w.editcol.list configure -width $maxcolwid
     }
+    $w.editcol.list configure -xscrollcommand "$w.editcol.xscroll set"
   }
 
-  update idletasks
-
   # Scroll to the top of the lists
   set fbbox [$flist bbox all]
   #gen_log:log D "fbbox   \"$fbbox\""
   if {[llength $fbbox] == 4} {
-    set botx [lindex $fbbox 0]
-    set boty [lindex $fbbox 1]
-    set topx [lindex $fbbox 2]
-    set topy [lindex $fbbox 3]
+    set ylen [expr {[lindex $fbbox 3] - [lindex $fbbox 1]}]
+
+    set wview [winfo height $flist]
+    $w.yscroll set 0 [expr ($wview * 1.0) / ($ylen * 1.0)]
 
     $flist config -scrollregion $fbbox
     $flist yview moveto 0
     
-    if {[winfo ismapped $w.statcol.list]} {
-      $w.statcol.list config -scrollregion \
-        [list $botx $boty [expr {$botx + $DirList($w:statcolwidth)}] $topy]
-      $w.statcol.list yview moveto 0
-    }
-    if {[winfo ismapped $w.datecol.list]} {
+    if {$cvscfg(showdatecol)} {
+      set fbbox [$w.datecol.list bbox all]
+      set botx [lindex $fbbox 0]
+      set boty [lindex $fbbox 1]
       $w.datecol.list config -scrollregion \
-        [list $botx $boty [expr {$botx + $DirList($w:datecolwidth)}] $topy]
+        [list $botx $boty \
+          [expr {$botx + $DirList($w:datecolwidth)}] \
+          [expr {$boty + $ylen}]]
       $w.datecol.list yview moveto 0
     }
-    if {[winfo ismapped $w.wrevcol.list]} {
-      $w.wrevcol.list config -scrollregion \
-        [list $botx $boty [expr {$botx + $DirList($w:revcolwidth)}] $topy]
-      $w.wrevcol.list yview moveto 0
-    }
-    if {[winfo ismapped $w.editcol.list]} {
-      $w.editcol.list config -scrollregion \
-        [list $botx $boty [expr {$botx + $DirList($w:edcolwidth)}] $topy]
-      $w.editcol.list yview moveto 0
-    }
-  }
-  update idletasks
-  if {[llength $fbbox] == 4} {
-    set whigh [expr abs([lindex $fbbox 1] - [lindex $fbbox 3])]
-    set wview [winfo height $flist]
-    set frac [expr ($wview * 1.0) / ($whigh * 1.0)]
-    update idletasks
-    $w.yscroll set 0 $frac
-  }
-  gen_log:log D "[array names DirList $w:*:selected]"
 
+    if {$incvs || $inrcs} {
+      set fbbox [$w.wrevcol.list bbox all]
+      set botx [lindex $fbbox 0]
+      set boty [lindex $fbbox 1]
+      $w.wrevcol.list config -scrollregion \
+        [list $botx $boty \
+          [expr {$botx + $DirList($w:revcolwidth)}] \
+          [expr {$boty + $ylen}]]
+      $w.wrevcol.list yview moveto 0
+
+      if {$cvscfg(showstatcol)} {
+        set fbbox [$w.statcol.list bbox all]
+        set botx [lindex $fbbox 0]
+        set boty [lindex $fbbox 1]
+        $w.statcol.list config -scrollregion \
+          [list $botx $boty \
+            [expr {$botx + $DirList($w:statcolwidth)}] \
+            [expr {$boty + $ylen}]]
+        $w.statcol.list yview moveto 0
+      }
+
+      if {$cvscfg(econtrol)} {
+        set fbbox [$w.editcol.list bbox all]
+        set botx [lindex $fbbox 0]
+        set boty [lindex $fbbox 1]
+        $w.editcol.list config -scrollregion \
+          [list $botx $boty \
+            [expr {$botx + $DirList($w:edcolwidth)}] \
+            [expr {$boty + $ylen}]]
+        $w.editcol.list yview moveto 0
+      }
+    }
+  }
+  #gen_log:log D "[array names DirList $w:*:selected]"
   gen_log:log T "LEAVE"
 }
 
@@ -942,7 +997,9 @@ proc DirCanvas:buildwhenidle {w} {
 }
 
 proc DirCanvas:scroll_windows {w args} {
-# Scrolling caused by arrow or page keys
+  global cvscfg
+  global incvs
+  global inrcs
 
   #gen_log:log T "ENTER ($w $args)"
   set way [lindex $args 1]
@@ -966,18 +1023,25 @@ proc DirCanvas:scroll_windows {w args} {
     }
   }
   eval $w.filecol.list yview $args
-  if {[winfo ismapped $w.statcol.list]} {
-    eval $w.statcol.list yview $args
+  if {$cvscfg(showdatecol)} {
+    eval $w.datecol.list yview $args
   }
-  eval $w.datecol.list yview $args
-  eval $w.wrevcol.list yview $args
-  if {[winfo ismapped $w.editcol.list]} {
-    eval $w.editcol.list yview $args
+  if {$incvs || $inrcs} {
+    eval $w.wrevcol.list yview $args
+    if {$cvscfg(showstatcol)} {
+      eval $w.statcol.list yview $args
+    }
+    if {$cvscfg(econtrol)} {
+      eval $w.editcol.list yview $args
+    }
   }
 }
 
 proc DirCanvas:drag_windows {w W y} {
 #Scrolling caused by dragging
+  global incvs
+  global inrcs
+  global cvscfg
 
   set height [$W cget -height]
   #gen_log:log D "$w %y $height"
@@ -986,10 +1050,18 @@ proc DirCanvas:drag_windows {w W y} {
   set yfrac [expr {double($y) / $height}]
 
   eval $w.filecol.list yview moveto $yfrac
-  eval $w.statcol.list yview moveto $yfrac
-  eval $w.datecol.list yview moveto $yfrac
-  eval $w.wrevcol.list yview moveto $yfrac
-  eval $w.editcol.list yview moveto $yfrac
+  if {$cvscfg(showdatecol)} {
+    eval $w.datecol.list yview moveto $yfrac
+  }
+  if {$incvs || $inrcs} {
+    eval $w.wrevcol.list yview moveto $yfrac
+    if {$cvscfg(showstatcol)} {
+      eval $w.statcol.list yview moveto $yfrac
+    }
+    if {$cvscfg(econtrol)} {
+      eval $w.editcol.list yview moveto $yfrac
+    }
+  }
 }
 
 proc DirCanvas:sort_by_col {w col sense} {
@@ -1024,6 +1096,7 @@ proc DirCanvas:sort_by_col {w col sense} {
 
 proc DirCanvas:toggle_col {w col} {
   global cvsglb
+  global cvscfg
 
   gen_log:log T "ENTER ($col)"
   set cur_col [lindex $cvsglb(sort_pref) 0]
@@ -1041,8 +1114,8 @@ proc DirCanvas:toggle_col {w col} {
     set sense "-decreasing"
   }
 
-  DirCanvas:sort_by_col $w $col $sense
   gen_log:log D "sort column $col $sense"
+  DirCanvas:sort_by_col $w $col $sense
 
   gen_log:log T "LEAVE"
 }
@@ -1085,16 +1158,16 @@ proc DirCanvas:makepopup {w} {
   $w.cvsdir_pop add command -label "Update with Options" \
     -command update_run
   $w.cvsdir_pop add command -label "Release" \
-    -command { cvs_release [workdir_list_files] }
+    -command { release_dialog [workdir_list_files] }
 
   # For CVS files
   menu $w.stat_ok_pop -tearoff 0
   $w.stat_ok_pop add command -label "Open" \
     -command { workdir_edit_file [workdir_list_files] }
   $w.stat_ok_pop add command -label "Log Browse" \
-    -command { cvs_logcanvas [workdir_list_files] }
+    -command { cvs_logcanvas [pwd] [workdir_list_files] }
   $w.stat_ok_pop add command -label "Annotate" \
-    -command { cvs_annotate [workdir_list_files] }
+    -command { cvs_annotate $current_tagname [workdir_list_files] }
   $w.stat_ok_pop add command -label "Remove" \
     -command { subtract_dialog [workdir_list_files] }
 
@@ -1113,9 +1186,9 @@ proc DirCanvas:makepopup {w} {
   $w.stat_merge_pop add command -label "Diff" \
     -command { cvs_diff [workdir_list_files] }
   $w.stat_merge_pop add command -label "Annotate" \
-    -command { cvs_annotate [workdir_list_files] }
+    -command { cvs_annotate $current_tagname [workdir_list_files] }
   $w.stat_merge_pop add command -label "Log Browse" \
-    -command { cvs_logcanvas [workdir_list_files] }
+    -command { cvs_logcanvas [pwd] [workdir_list_files] }
 
   # For CVS files that are modified
   menu $w.stat_mod_pop -tearoff 0
@@ -1140,13 +1213,15 @@ proc DirCanvas:makepopup {w} {
   $w.stat_conf_pop add command -label "Merge Conflict" \
     -command { cvs_merge_conflict [workdir_list_files] }
   $w.stat_conf_pop add command -label "Annotate" \
-    -command { cvs_annotate [workdir_list_files] }
+    -command { cvs_annotate $current_tagname [workdir_list_files] }
   $w.stat_conf_pop add command -label "Log Browse" \
-    -command { cvs_logcanvas [workdir_list_files] }
+    -command { cvs_logcanvas [pwd] [workdir_list_files] }
 
-  menu $w.updir_pop -tearoff 0
-  $w.updir_pop add command -label "Open" \
+  menu $w.rcs_pop -tearoff 0
+  $w.rcs_pop add command -label "Open" \
     -command { workdir_edit_file [workdir_list_files] }
+  $w.rcs_pop add command -label "Delete" \
+    -command { workdir_delete_file [workdir_list_files] }
 
   gen_log:log T "LEAVE"
 }
