@@ -77,6 +77,7 @@ namespace eval ::exec {
         variable errmsg
         variable data
         variable v_w
+        variable my_idx
         global errorCode
       
         # Blocking read -- returns -1 on EOF.  Then you get the process return
@@ -88,10 +89,17 @@ namespace eval ::exec {
             set ExecDone [list 1 $res $errorCode]
             gen_log:log E "  ExecDone $ExecDone"
             if {$errmsg == ""} { set errmsg $res }
-            # Since we don't pop up an error dialog, let's at least try to show
+
+            [namespace current]::err_handler
+
+            if {$errmsg == ""} {set errmsg "$command exited status $status"}
+            if {[string length $errmsg] < 256} {
+               cvsfail $errmsg
+            }
+            # If we don't pop up an error dialog, let's at least try to show
             # what happened in the viewer window, if there is one
-            if {$viewer != ""} {
-               $v_w.text insert end "$res\n" stderr
+            if {$viewer != {}} {
+               $v_w.text insert end "\n$res" stderr
                if {[tell $procerr]} {
                  seek $procerr 0
                  while {[gets $procerr erline] != -1} {
@@ -99,10 +107,17 @@ namespace eval ::exec {
                  }
                }
              }
+             ::exec::$my_idx\::abort
           } else {
             gen_log:log D "  Close OK"
             set ExecDone [list 0]
             gen_log:log D "  ExecDone $ExecDone"
+            
+          }
+          if {$viewer != {}} {
+            pack forget $v_w.stop
+            pack $v_w.close -in $v_w.bottom -side right -ipadx 15
+            $v_w.close configure -state normal
           }
           return
         }
@@ -135,51 +150,49 @@ namespace eval ::exec {
         variable show_stderr
         variable v_w
 
-        #SRIV this is the key to no cpu usage!
-        fileevent $procerr readable {}
-
         # When new stuff appears in the error output file, get it.  There may
         # be more than one line.
         set errmsg ""
         if {[tell $procerr] != $errpos} {
           seek $procerr $errpos start
           while {[gets $procerr erline] != -1} {
-            append errmsg "$erline"
+            append errmsg "\n$erline"
             set errpos [tell $procerr]
           }
           gen_log:log E "$errmsg"
           if {$viewer != "" && $show_stderr == 1} {
-            $v_w.text insert end "$errmsg\n" stderr
+            $v_w.text insert end "\n$errmsg" stderr
           }
         }
-        
-        #SRIV this is the key to no cpu usage!
-        #after 1 [list catch {fileevent $procerr readable [namespace current]::err_handler}]
-        catch {fileevent $procerr readable [namespace current]::err_handler}
+
       }
 
       proc abort {} {
         variable procout
         variable procerr
         variable procid
-        variable ExecDone
+        variable viewer
+        variable v_w
 	global tcl_platform
 
         gen_log:log T "ENTER"
-#puts "aborting [namespace current]"
         # This does the trick but it wont work on windows
         if {![info exists procid]} {
           gen_log:log D "procid is not defined"
-#puts "procid is not defined"
           return
         }
         catch "exec kill $procid" kres
         unset procid
-#puts "$kres"
+
+        err_handler
+        if {$viewer != {}} {
+          pack forget $v_w.stop
+          pack $v_w.close -in $v_w.bottom -side right -ipadx 15
+          $v_w.close configure -state normal
+        }
+
         catch {close $procout} cres
-#puts "$cres"
         catch {close $procerr} cres
-#puts "$cres"
         gen_log:log D "$kres"
 
         gen_log:log T "LEAVE"
@@ -187,20 +200,10 @@ namespace eval ::exec {
 
       proc wait {} {
         variable ExecDone
-        variable procid
-        variable my_idx
         gen_log:log T "ENTER"
 
-#puts "wait: [namespace current]"
         if {!$ExecDone} {
-#puts " waiting for [namespace current]::ExecDone"
           vwait [namespace current]::ExecDone
-          set prev [expr $my_idx - 1]
-          if {[info exists ::exec::$prev\::procid]} {
-#puts "Aborting previous exec"
-            ::exec::$prev\::abort
-          }
-#puts "end wait [namespace current]::ExecDone\n"
         }
         gen_log:log T "LEAVE"
       }
@@ -224,7 +227,6 @@ namespace eval ::exec {
         variable procerr
         variable procid
         variable errmsg
-        variable ExecDone
         variable command
         variable viewer
         variable filter
@@ -243,7 +245,6 @@ namespace eval ::exec {
 
         # Here's where we do it
         gen_log:log C "$command"
-#puts " [namespace current] run_exec $command"
         set procout [open "| $command 2>@$procerr" r]
         set procid [pid $procout]
         # Dont ever do this.  The whole thing depends on procout blocking
@@ -253,37 +254,9 @@ namespace eval ::exec {
         flush $procerr
         fileevent $procerr readable [list [namespace current]::err_handler]
   
-        vwait [namespace current]::ExecDone
-
-        set status $ExecDone
-#puts " finished [namespace current]"
-        # If you get two execs going at once, the first one does not come
-        # out if its wait and you hang forever.  This is the only way I
-        # could come up with to keep from hanging all the time.
-        set prev [expr $my_idx - 1]
-        if {[info exists ::exec::$prev\::procid]} {
-#puts "Aborting previous exec"
-          ::exec::$prev\::abort
-        }
-        gen_log:log D "Finish vwait"
-
         # set buffering back to normal
         fconfigure stdout -blocking true -buffering line
         catch {fileevent $procerr readable {} }
-    
-        if {$viewer != {}} {
-          set v_w [namespace inscope $viewer {set w}]
-          pack forget $v_w.stop
-          pack $v_w.close -in $v_w.bottom -side right -ipadx 15
-          $v_w.close configure -state normal
-        }
-        update
-        if {$ExecDone != 0} {
-          bell
-          #if {$errmsg == ""} {set errmsg "$command exited status $status"}
-          #cvsfail $errmsg
-        }
-        namespace delete [namespace current]
       }
 
       after 0 [list [namespace current]::run_exec]
@@ -515,9 +488,10 @@ proc viewer_window {w title parent} {
   scrollbar $w.scroll -relief sunken -command "$w.text yview"
   frame $w.bottom
   button $w.bottom.srchbtn -text Search -command "$parent\::search"
-  
   entry $w.bottom.entry -width 20 -textvariable searchstr
   bind $w.bottom.entry <Return> "$parent\::search"
+  
+  button $w.save -text "Save to File" -command "save_viewcontents $w"
   button $w.close -text "Close" -command "
     namespace delete $parent
     destroy $w
@@ -531,9 +505,28 @@ proc viewer_window {w title parent} {
   pack $w.text -fill both -expand 1
   pack $w.bottom.srchbtn -side left
   pack $w.bottom.entry -side left
+  pack $w.save -in $w.bottom -side left -padx 25
   pack $w.close -in $w.bottom -side right -ipadx 15
 
   # Focus to activate text bindings
   focus $w
   wm title $w "$title"
 }
+
+proc save_viewcontents {w} {
+  set types  { {{All Files} *} }
+  set savfile [ \
+    tk_getSaveFile -title "Save Results Summary" \
+       -initialdir "." \
+       -filetypes $types \
+  ]  
+  if {$savfile == ""} {
+    return
+  } 
+  if {[catch {set fo [open $savfile w]}]} {
+    puts "Cannot open $savfile for writing"
+    return
+  }
+  puts $fo [$w.text get 1.0 end]
+  close $fo
+}
