@@ -3,7 +3,7 @@
 #
 
 #
-# $Id: filebrowse.tcl,v 1.12 2002/01/23 06:55:26 dorothyr Exp $
+# $Id: filebrowse.tcl,v 1.18 2003/10/17 07:30:11 dorothyr Exp $
 #
 # Sets up a dialog to browse the contents of a module.
 #
@@ -64,9 +64,10 @@ proc browse_files {module} {
   #
   # Create buttons
   #
-  button $filebrowse.help -text "Help" \
-    -padx 0 -pady 0 \
-    -command file_browser
+  if {[catch "image type Fileview"]} {
+    workdir_images
+  }
+
   button $filebrowse.view -image Fileview \
     -command "fileview $filebrowse $module"
   button $filebrowse.log -image Branches \
@@ -77,8 +78,6 @@ proc browse_files {module} {
     -padx 0 -pady 0 \
     -command "destroy $filebrowse"
 
-  pack $filebrowse.help \
-    -in $filebrowse.down -side left -ipadx 0 -ipady 0 -fill both -expand 1
   pack $filebrowse.view \
        $filebrowse.log \
        $filebrowse.tag \
@@ -127,15 +126,17 @@ proc browse_files {module} {
 proc filepath {module filename} {
   # Prepend a path to the filename if needed
   global modval
-  global mod_tagA
+  global module_dir
   global cvscfg
   global cvs
 
   gen_log:log T "ENTER ($filename $module)"
   regsub -all {\$} $filename {\$} file
  
+  # set global module variable - logcanvas may need it
+
   set commandline \
-     "$cvs -d $cvscfg(cvsroot) rdiff -s -D 01/01/1971 -r $mod_tagA \"$file\""
+     "$cvs -d $cvscfg(cvsroot) rdiff -s -D 01/01/1971 \"$file\""
   gen_log:log C  $commandline
   set ret [catch {eval "exec $commandline"} view_this]
   gen_log:log D "\"$view_this\""
@@ -146,9 +147,9 @@ proc filepath {module filename} {
 
   if {[info exists modval($module)]} {
     gen_log:log D "modval $module \"$modval($module)\""
-    set modname $modval($module)
-    #set file "$modname/[file tail $file]"
-    set file "$modname/$file"
+    set module_dir $modval($module)
+    #set file "$module_dir/[file tail $file]"
+    set file "$module_dir/$file"
     gen_log:log T "LEAVE (prepend modval) ($file)"
     return $file
   }
@@ -175,7 +176,7 @@ proc fileview {toplevelname module} {
   foreach item [$listname curselection] {
     set v [$listname get $item]
     set f [filepath $module $v]
-    cvs_fileview "$f" [$toplevelname.tver1 get]
+    cvs_fileview_checkout [$toplevelname.tver1 get] "$f"
   }
   gen_log:log T "LEAVE"
 }
@@ -187,7 +188,7 @@ proc tagview {toplevelname module} {
   foreach item [$listname curselection] {
     set v [$listname get $item]
     set f [filepath $module $v]
-    view_output "$f Tags" [cvs_gettaglist "$f"]
+    view_output::new "$f Tags" [cvs_gettaglist "$f"]
   }
   gen_log:log T "LEAVE"
 }
