@@ -3,7 +3,7 @@
 #
 
 #
-# $Id: import.tcl,v 1.45 2004/04/05 16:35:11 dorothyr Exp $
+# $Id: import.tcl,v 1.46 2004/11/15 06:08:35 dorothyr Exp $
 #
 # Adds a new document to the repository.
 #
@@ -12,9 +12,6 @@ proc import_run {} {
   global cwd
   global incvs
   global cvsglb
-  global cvscfg
-  global modlist
-  global modlist_sorted
 
   gen_log:log T "ENTER"
   
@@ -120,13 +117,11 @@ proc do_import {} {
     cvsok "You must type in a new module path directory."
     return 1
   }
-  if { $cvsglb(newvers) != "" } {
-    if {[llength [split  $cvsglb(newvers) {.}]] != 3} {
-      cvsok "CVS only allows branches with two dots."
-      return 1
-    }
-  }
   
+  # We may have gotten here before opening the module browser
+  if {! [info exists modlist_sorted]} {
+     modbrowse_run
+  }
   # Make sure it isn't a duplicate key
   foreach {key value} [array get modval] {
      if { $cvsglb(newcode) == $key } {
@@ -135,7 +130,6 @@ proc do_import {} {
      }
   }
   
-
   # See if all apropriate Directories in newdirname exist.  CVS import will
   # create them, but we'll want to make a #D entry.
   set cvsglb(newdir) [string trimleft $cvsglb(newdir) "/"]
@@ -152,12 +146,14 @@ proc do_import {} {
   }
 
   # Make a baseline tag
-  set baseline [join [concat "baseline" [split $cvsglb(newvers) {.}]] {_} ]
+  set versions [split $cvsglb(newvers) ".,/ -"]
+  set baseline "baseline-[join $versions {_}]"
 
   set commandline "$cvs -d $cvscfg(cvsroot) import -m \"Imported using TkCVS\""
-  if {$cvsglb(newvers) != ""} {
-    append commandline " -b $cvsglb(newvers)"
-  }
+  # Let it default to 1.1.1 or you will have big problems later from cvs.
+  #if {$cvsglb(newvers) != ""} {
+    #append commandline " -b 1.1.1"
+  #}
   append commandline " \"$cvsglb(newdir)\" IMPORT $baseline"
 
   set v [viewer::new "Import Module"]
