@@ -624,7 +624,7 @@ proc cvs_joincanvas { } {
   set files [glob -nocomplain -types f -- .??* *]
 
   regsub -all {\$} $files {\$} files
-  set commandline "$cvs -d $cvscfg(cvsroot) -n log -l $files"
+  set commandline "$cvs -d $cvscfg(cvsroot) log $files"
   gen_log:log C "$commandline"
   catch {eval "exec $commandline"} raw_log
   set log_lines [split $raw_log "\n"]
@@ -681,7 +681,7 @@ proc join_getlog {filename {name_idx {}}} {
   global current_tagname
 
   gen_log:log T "ENTER ($filename $name_idx)"
-  set commandline "$cvs -d $cvscfg(cvsroot) -n log -l \"$filename\""
+  set commandline "$cvs -d $cvscfg(cvsroot) log \"$filename\""
   gen_log:log C "$commandline"
   set ret [catch {eval "exec $commandline"} view_this]
   if {$ret} {
