@@ -3,7 +3,7 @@
 #
 
 #
-# $Id: logcanvas.tcl,v 1.70 2004/03/12 07:37:05 dorothyr Exp $
+# $Id: logcanvas.tcl,v 1.72 2004/11/08 04:58:56 dorothyr Exp $
 #
 # Contains procedures used for the log canvas for tkCVS.
 #
@@ -260,18 +260,33 @@ namespace eval ::logcanvas {
             {D} {
               # Look for a date line.  This also has the name of the author.
               set parts [split $logline]
-              foreach [list \
-                  revwho($revnum) revdate($revnum) revtime($revnum) \
-                  revlines($revnum) revstate($revnum) \
-                ] \
-                [list \
-                  [string trimright [lindex $parts 5] {;}] \
-                  [lindex $parts 1] \
-                  [string trimright [lindex $parts 2] {;}] \
-                  [lrange $parts 11 end] \
-                  [string trimright [lindex $parts 8] {;}] \
-                ] \
-                { break }
+	      if {[lindex $parts 4] == "author:"} {
+                foreach [list \
+                    revwho($revnum) revdate($revnum) revtime($revnum) \
+                    revlines($revnum) revstate($revnum) \
+                  ] \
+                  [list \
+                    [string trimright [lindex $parts 5] {;}] \
+                    [lindex $parts 1] \
+                    [string trimright [lindex $parts 2] {;}] \
+                    [lrange $parts 11 end] \
+                    [string trimright [lindex $parts 8] {;}] \
+                  ] \
+                  { break }
+	      } else {
+                foreach [list \
+                    revwho($revnum) revdate($revnum) revtime($revnum) \
+                    revlines($revnum) revstate($revnum) \
+                  ] \
+                  [list \
+                    [string trimright [lindex $parts 6] {;}] \
+                    [lindex $parts 1] \
+                    [string trimright [lindex $parts 2] {;}] \
+                    [lrange $parts 11 end] \
+                    [string trimright [lindex $parts 8] {;}] \
+                  ] \
+                  { break }
+	      }
               set logstate {L}
             }
             {L} {
@@ -1276,7 +1291,7 @@ namespace eval ::logcanvas {
                     }
                     $logcanvas.canvas create line \
                       $xfrom $yfrom $xmid $ymid $xto $yto \
-                      -arrow first -smooth bezier
+                      -arrow first -smooth 1
                  }
               }
             }
