@@ -1,5 +1,5 @@
 #
-# $Id: modbrowse.tcl,v 1.49 2003/11/05 06:37:28 dorothyr Exp $
+# $Id: modbrowse.tcl,v 1.51 2004/03/12 07:37:05 dorothyr Exp $
 #
 # Set up a check out dialog.
 #
@@ -9,6 +9,7 @@ proc modbrowse_setup {} {
   global modbrowse_module
   global cvsglb
   global cvscfg
+  global tcl_platform
 
   gen_log:log T "ENTER"
   set cwd [pwd]
@@ -17,7 +18,9 @@ proc modbrowse_setup {} {
   toplevel .modbrowse
   wm title .modbrowse "Module Browser"
   wm iconname .modbrowse "CVS Modules"
-  wm iconbitmap .modbrowse @$cvscfg(bitmapdir)/tkcvs48.xbm
+  if {$tcl_platform(platform) != "windows"} {
+    wm iconbitmap .modbrowse @$cvscfg(bitmapdir)/tkcvs48.xbm
+  }
   wm protocol .modbrowse WM_DELETE_WINDOW {
     .modbrowse.bottom.buttons.closefm.close invoke 
   }
@@ -234,8 +237,10 @@ proc modbrowse_menus {} {
   .modbrowse.modmenu.file add command -label "Module File" \
      -command module_file
   .modbrowse.modmenu.file add separator
-  .modbrowse.modmenu.file add command -label "Import" -underline 0 \
+  .modbrowse.modmenu.file add command -label "Import To A New Module" -underline 0 \
      -command { import_run }
+  .modbrowse.modmenu.file add command -label "Import To An Existing Module" -underline 0 \
+     -command { import2_run }
   .modbrowse.modmenu.file add command -label "Vendor Merge" -underline 0 \
      -command {merge_run $modbrowse_module}
   .modbrowse.modmenu.file add separator
