@@ -3,7 +3,7 @@
 #
 
 #
-# $Id: workdir.tcl,v 1.140 2004/03/14 03:44:08 dorothyr Exp $
+# $Id: workdir.tcl,v 1.141 2004/11/08 04:39:10 dorothyr Exp $
 #
 # Current working directory display.  Handles all of the functions
 # concerned with navigating about the current directory on the main
@@ -417,6 +417,10 @@ proc workdir_menus {} {
      -command { subtract_dialog [workdir_list_files] }
   .workdir.menubar.cvs add command -label "Remove Recursively" \
      -command { subtractdir_dialog [workdir_list_files] }
+  .workdir.menubar.cvs add command -label "Set Binary Flag" \
+     -command { cvs_binary [workdir_list_files] }
+  .workdir.menubar.cvs add command -label "Set ASCII Flag" \
+     -command { cvs_ascii [workdir_list_files] }
   .workdir.menubar.cvs add command -label "Set Edit Flag (Edit)" -underline 15 \
      -command { cvs_edit [workdir_list_files] }
   .workdir.menubar.cvs add command -label "Reset Edit Flag (Unedit)" -underline 11 \
